#!/usr/bin/env python
#coding=utf8
"""
$Id: DS18B20.py, 2017/01/21
Copyright (c) 2014 C-Bell (VAGor). 
Чтение значений температур с цифровых термометров DS18B20
по шине 1-wire, подключенной к GPIO4 одноплатного компьютера
Raspberry PI

Условия:
- к GPIO4 подключена шина 1-wire с  резистором 4,7 kOm,
  второй вывод резистора подключен к выводу 3,3V
- в конфигураторе Raspberry PI должна быть включена шина 1-wire
  (установлены и запущены модули w1-gpio и w1-therm)

Использование:
import DS18B20
(DS18B20list, tMeasure)=DS18B20.Measure()
где DS18B20list - список кортежей (ID термометра, значение температуры)
    tMeasure    - момент фиксации показаний термометров

Особенности:
используется модуль multiprocessing.dummy для "одновременного"
запуска измерения температыры всеми цифровыми термометрами DS18B20
что в большинстве случаев сокращает до минимума время опроса термометров
"""

import os       #Модуль функций для работы с операционной системой
import sys      #Модуль доступа к некоторым системным переменным и функциям
import glob     #находит все пути, совпадающие с заданным шаблоном 
import time     #Модуль для работы со временем
from multiprocessing.dummy import Pool as ThreadPool

#os.system('modprobe w1-gpio')
#os.system('modprobe w1-therm')

#Класс исключения отсутствия наличия шины 1-wire
class NoOneWireError(Exception):
    pass

#Класс исключения при отсутствии цифровых термометров на шине 1-wire
class NoDS18B20Error(Exception):
    pass

#Проверка наличия шины 1-wire
if not os.path.isdir('/sys/bus/w1'):
    #Генерить исключение.
    raise NoOneWireError('No 1-wire bus')
    print("Включите поддержку 1-Wire в  конфигурации Raspberry!")
    exit()
    
#Каталог, отражающий устройства на шине 1-wire
base_dir = '/sys/bus/w1/devices/'


#Читает значение температуры с указанного термометра и возвращает его
def read_temp_raw(device_folder):
    '''read_temp_raw читает значение температуры с указанного
    термометра и возвращает кортеж (ID термометра, температура
    в градусах цельсия)'''
    
    #полное имя файла, из которого читается температура
    device_file = device_folder + '/w1_slave'

    # создать объект-файл
    f = open(device_file, 'r')

    #прочитать строки из файла
    lines = f.readlines()

    #закрыть файл
    f.close()

    #найти сочетание символов t=
    equals_pos = lines[1].find('t=')

    #вычленить температуру из строки
    temp_string = lines[1][equals_pos+2:]

    #преобразовать в число
    temp_c = float(temp_string) / 1000.0

    #вернуть прочитанные данные
    return (os.path.basename(device_folder), temp_c)


#Возвращает кортеж из списка кортежей названий термометров
#с температурами и временем измерения
def Measure():
    '''Функция Measure() ищет все цифровые термометры DS18B20 на шине 1-wire
    читает значения измеренных ими температур и возвращает в виде кортежа
    (список кортежей (ID термометра, температура), момент времени измерения)'''
    device_folders = glob.glob(base_dir + '28*')

    #Если папки DS18B20 не обнаружены, сформировать исключение
    if len(device_folders)==0:
        raise NoDS18B20Error('Digital thermometers DS18B20 not found')

    #задать количество процессов равным количеству DS18B20
    pool = ThreadPool(len(device_folders))

    #запустить параллельное измерение и получение температуры
    DS18B20list=pool.map(read_temp_raw, device_folders)

    #закрать мультипроцессинг
    pool.close()

    #присоединить к основному потоку, чтобы он ждал завершение измерения
    pool.join()
    return (DS18B20list, time.time()) #выдать результаты измерения

def main():
    '''Тестовая функция отображает результаты измерения
    подсчитывает среднее время измерения и отображает
    максимальное время измерения.'''
    t=0         #общее время измерения
    tMax=0      #максимальная продолжительность измерения
    for i in xrange(1, sys.maxsize):
        timeBegin=time.time()
        (DS18B20list, tMeasure)=Measure()
        os.system('clear')
        for DS18B20 in DS18B20list:
            print(u"%s\t%3.1f градC" % DS18B20)
        print(u'Время измерения=%ssec' % (tMeasure-timeBegin))
        t+=(tMeasure-timeBegin)
        print(u'Среднее время измерения=%ssec' % (t/i))
        if (tMax<(tMeasure-timeBegin)):
              tMax=(tMeasure-timeBegin)
        print(u'(mp)Максимальная продолжительность измерения=%ssec' % tMax)
        print('')

if __name__ == '__main__':
    main()

        
